'' =================================================================================================
''
''   File....... hc-8+_template__2016-01-07.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2011-2016 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... teamefx@efx-tek.com 
''   Started.... 
''   Updated.... 07 JAN 2016
''
'' =================================================================================================


con

  ' Important!!! If you use this template with earlier PCBs, you MUST changes the revision constant
  ' before downloading.

  PCB_REV = "F"


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
                                                                 
  CLK_FREQ = _xinfreq * (_clkmode >> 6)                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
                                                                 
  EFX_BAUD =  38_400                                             
  TRM_BAUD = 115_200                                             
                                                                 
                                                                 
con { io pins }                                                  
                                                                 
  RX1     = 31  { I }                                           ' serial / programming
  TX1     = 30  { O }                                            
                                                                 
  SDA     = 29  { I/O }                                         ' i2c / eeprom
  SCL     = 28  { I/O }                                          
                                                                 
  OUT_EN  = 27  { O }   { XP0 on Rev F pcb }                    ' outputs enable pin (low)     
                                                                 
  XP0     = 27  { I/O } { OUT_EN on Rev E pcb }                 ' extra port 0 - not on Rev E or lower
  XP1     = 24  { I/O }                                         ' extra port 1 - not on Rev E or lower 
  XP2     = 19  { I/O }                                         ' extra port 2 - not on Rev E or lower    
  XP3     = 16  { I/O }                                         ' extra port 3 - not on Rev E or lower 
                                                                 
  SIO_RX  = 26  { I/O }                                         ' TTL serial io
  SIO_TX  = 25  { I/O }                                         ' TTL serial io (not enabled)  
                                                                 
  RX2     = 23  { I }                                           ' RS-485 port
  RX2_EN  = 22  { O }                                                
  TX2_EN  = 21  { O }                                                
  TX2     = 20  { O }                                                
                                                                 
  R_LED   = 18  { O }                                           ' bi-color LED
  G_LED   = 17  { O }                                                
                                                                 
  OUT7    = 15  { O }                                           ' outputs
  OUT6    = 14  { O }                                                
  OUT5    = 13  { O }                                                
  OUT4    = 12  { O }                                                
  OUT3    = 11  { O }                                                
  OUT2    = 10  { O }                                                
  OUT1    =  9  { O }                                                
  OUT0    =  8  { O }                                                
                                                                 
  OPT_BR  =  7  { I/O }                                         ' options switches
  OPT_A1  =  6  { I/O }                                                 
  OPT_A0  =  5  { I/O }                                                 
  OPT_SM  =  4  { I/O }                                                 
                                                                 
  DMX_A8  =  3  { I }                                           ' high-bit of DMX address
                                                                 
  DO_165  =  2  { I }                                           ' control lines for 74x165
  CLK_165 =  1  { O }                                                 
  LD_165  =  0  { O }                                                 
                                                                 
                                                                 
  ' for EFX-TEK GameOn adapter module (#99001)                   
  ' -- must open A0 and SM option switches when installed        
                                                                 
  WII_DAT =  OPT_A0                                              
  WII_CLK =  OPT_SM                                              

  
  ' for EFX-TEK uSD adapter module (#99002)
  ' -- must open all option switches when installed

  SD_CS   =  OPT_BR                                       
  SD_DI   =  OPT_A1  
  SD_CLK  =  OPT_A0  
  SD_DO   =  OPT_SM


dat

  XPort         byte    XP0, XP1, XP2, XP3                      ' XPort pin mapping
  
  
con

  #-1, ON, OFF                                                  ' digital control
                                                                 
  #0, BLK, RED, GRN, YEL                                        ' for r/g led
                                                                 
  #0, NOCON, RX, TX, RXTX                                       ' for rs-485
                                                                 
  #0, LSBFIRST, MSBFIRST                                         
                                                                 
                                                                 
obj                                                              
                                                                 
' main                                                          ' * master Spin cog
  time       : "jm_time"                                        '   timing and delays
  prng       : "jm_prng"                                        '   pseudo-random number generator
  io         : "jm_io_basic"                                    '   essential io
  pwm        : "jm_pwm8"                                        ' * pwm driver for outputs
  fader      : "jm_fade_manager"                                ' * manage up to 8 simultaneous fades 
  serial     : "jm_fullduplexserial"                            ' * serial IO for AP-16+, HC-8+, FC-4+, etc.
' background                                                    ' * background RGY LED control, inputs scanning
                                                                 
' * uses cog when loaded
                                                             
                                                                  
var


pub main  
                                                                  
  setup                                                         ' setup io and objects
                                                                 
  repeat                                                         
    waitcnt(0)                                                  ' your code here
                                                                         
                                                                 
pub setup                                                        
                                                                 
'' Configure HC-8+ IO and support objects                        
                                                                 
  time.start                                                    ' setup for timing/delays
                                                                 
  prng.seed(cnt, cnt ~> 2, $EF8_7E4, cnt << 2, cnt)             ' setup randomizer                             
                                                                 
  io.start(0, 0)                                                ' clear pins (this cog)
                                                                 
  outa[OUT7..OUT0] := %0000_0000                                ' clear outputs
  dira[OUT7..OUT0] := %1111_1111                                ' output mode
                                                                 
  if (PCB_REV =< "E")                                            
    io.low(OUT_EN)                                              ' enable output driver    
                                                                 
  pwm.start(8, OUT0)                                            ' start outputs driver *
                                                                 
  fader.start(8, pwm.address)                                   ' start fader manager *
                                                                 
  set_rs485(NOCON)                                              ' RS-485 not connected
                                                                 
  ap16_setup                                                    ' start serial for AP-16+ *
                                                                 
  bcog := cognew(background, @bcstack) + 1                      ' start background cog *
                                                                 
                                                                 
con                                                              
                                                                 
  { --------------------------------- }                          
  {  S U P P O R T   R O U T I N E S  }                          
  { --------------------------------- }                          
                                                                 
                                                                 
var                                                              
                                                                 
  long  ttlpins                                                 ' updated by background cog
  long  dmxaddr                                                  
                                                                 
                                                                 
pub ttl_pin | btns                                               
                                                                 
'' Reads TTL states for 1-of-8 input                             
'' -- returns -1 (no input)                                      
'' -- returns 0 to 7 for input on IN0 to IN7                     
''    * only one input allowed                                   
                                                                 
  if (bit_count(ttlpins) == 1)                                  ' only one button pressed?
    return bit_pos(ttlpins, 0)                                  '  yes, return button #
  else                                                           
    return -1                                                   '  none or bad input
                                                                 
                                                                 
pub ttl_inputs                                                   
                                                                 
'' Return TTL inputs status                                      
'' -- pins are scanned by background cog                         
                                                                 
  return ttlpins                                                 
                                                                 

pub dmx_address 

'' Return DMX address switch setting
'' -- switches are scanned by background cog
'' -- Note: For Rev C and lower boards, do not use DMX 256 switch -- leave it OFF

  return dmxaddr


pub option_inputs

'' Returns state of option switch bits (%0000..%1111, all off to all on)

  dira[OPT_BR..OPT_SM] := %0000                                 ' force to input mode
                                                                 
  return !ina[OPT_BR..OPT_SM] & $0F                             ' return state (1 = on)
                                                                 
                                                                 
pub read_xport | xp                                             ' Rev F and higher only
                                                                 
'' Returns state of xport bits
'' -- does not set dira bits to 0

  if (PCB_REV < "F")
    return %0000
  else
    return (ina[XP3] << 3) | (ina[XP2] << 2) | (ina[XP1] << 1) | ina[XP0]


pub write_xport(value, outsmask) | idx, mask                    ' Rev F and higher only    
                                                                 
'' Write value to xport pins                                     
'' -- outsmask used to set xport output pins                     
'' -- will not change pins with 0 bit in outsmask                
                                                                 
  if (PCB_REV < "F")                                             
    return                                                       
                                                                 
  repeat idx from 0 to 3                                         
    mask := |<idx                                               ' create pin mask
    if (outsmask & mask)                                        ' write to this pin?
      if (value & mask)                                         ' pin high?
        io.high(XPort[idx])                                      
      else                                                       
        io.low(XPort[idx])                                       
                                                                 
                                                                 
pub set_rg_led(state)                                             
                                                                 
'' Sets R/G LED                                                  
'' -- direct control only                                        
'' -- red/green/yellow + blinking requires support cog           
                                                                 
  ifnot (bcog)                                                  ' if support cog not running
    dira[R_LED..G_LED] := %11                                   ' make outputs
    case state                                                  ' set color to state
      OFF : outa[R_LED..G_LED] := %00                            
      RED : outa[R_LED..G_LED] := %10                            
      GRN : outa[R_LED..G_LED] := %01                            
      YEL : outa[R_LED..G_LED] := %00                           ' yellow requires active driver
                                                                 
  else
    dira[R_LED..G_LED] := %00                                   ' release cog 0 pins                                                         
    case state                                                   
      OFF : set_rgy_led(OFF, 100, OFF, 100)                     ' set to solid color 
      RED : set_rgy_led(RED, 100, RED, 100)                          
      GRN : set_rgy_led(GRN, 100, GRN, 100)                          
      YEL : set_rgy_led(YEL, 100, YEL, 100)                      
                                                                 
                                                                 
pub set_rgy_led(p0c, p0ms, p1c, p1ms)                                
                                                                 
'' Explicit (control registers) setup of OGRY LED                
'' -- requires support cog to be running                         
'' -- p0c is phase 0 color (OFF, GRN, RED, YEL)                  
'' -- p0ms is phase 0 timing in milliseconds                     
'' -- p1c is phase 1 color                                       
'' -- p1ms is phase 1 timing in milliseconds                     
                                                                 
  ifnot (bcog)                                                   
    return                                                       
                                                                 
  if ((p0c => OFF) and (p0c =< YEL))                            ' phase 0 color
    rgycolor[0] := p0c                                           
  else                                                           
    rgycolor[0] := OFF                                           
                                                                 
  if (p0ms > 0)                                                 ' phase 0 timing                                           
    rgytime[0] := p0ms                                           
  else                                                           
    rgytime[0] := 100                                            
                                                                 
  if ((p1c => OFF) and (p1c =< YEL))                            ' phase 1 color                             
    rgycolor[1] := p1c                                           
  else                                                           
    rgycolor[1] := OFF                                           
                                                                 
  if (p1ms > 0)                                                 ' phase 1 timing                                              
    rgytime[1] := p1ms                                           
  else                                                           
    rgytime[1] := 100                                            
                                                                 
                                                                 
pub set_rs485(state)                                             
                                                                 
'' Sets RS-485 RX and TX enable inputs                           
                                                                 
  case state                                                     
    NOCON : outa[RX2_EN..TX2_EN] := %10                          
    RX    : outa[RX2_EN..TX2_EN] := %00                          
    TX    : outa[RX2_EN..TX2_EN] := %11                          
    RXTX  : outa[RX2_EN..TX2_EN] := %01                          
                                                                 
  dira[RX2_EN..TX2_EN] := %11                                    
                                                                 
                                                                 
pub bit_count(value) | bc                                        
                                                                 
'' Returns # of bits set (1) in value                            
                                                                 
  bc := 0                                                       ' clear count
  repeat 32                                                     ' test all bits
    bc += (value & %1)                                          ' add bit value to count
    value >>= 1                                                 ' next bit
                                                                 
  return bc                                                      
                                                                 
                                                                 
pub bit_pos(value, mode)                                         
                                                                 
'' Returns position of 1st bit set (0..31)                       
'' -- mode 0 to scan from lsb, mode 1 to scan from msb           
'' -- -1 = no bits set                                           
                                                                 
  if (value == 0)                                               ' if no bits                                   
    return -1                                                   '  return -1
  else                                                           
    if (mode == LSBFIRST)                                       ' check from LSB
      value ><= 32                                              '  flip for >|
      return (32 - >|value)                                      
    else                                                         
      return (>|value - 1)                                       
                                                                 
                                                                 
pub bit_val(value, pos)                                          
                                                                 
'' Returns bit (0..1) from pos (0..31) in value                  
                                                                 
  if ((pos => 0) and (pos =< 31))                                
    return (value >> pos) & 1                                    
  else                                                           
    return 0                                                     
                                                                 
                                                                 
con                                                              
                                                                 
  { ------------------------------- }                            
  {  A P - 1 6 +   R O U T I N E S  }                            
  { ------------------------------- }                            
                                                                 
                                                                 
dat                                                              
                                                                 
  AP16          byte    "!AP16", 0
 

pub ap16_setup

'' Start serial driver on SIO at EFX_BAUD (half-duplex true mode)
'' -- requires serial object defined in project
''    * object must be capable of half-duplex comms over a single wire
''    * assumes board has not been modified for 2-wire serial

  serial.start(SIO_RX, SIO_RX, %1100, EFX_BAUD)
  time.pause(5)
  serial.rxflush


pub ap16_reset(addr)

'' Stops AP-16+ at addr
'' -- addr is %00..%11
''    * use $FF for all AP-16+s

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("X")


pub ap16_status(addr)

'' Returns AP-16+ status byte
'' -- addr is %00..%11

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("G")
  serial.rxflush

  return serial.rxtime(25)


pub ap16_playing(addr) | check

'' Returns playing status from AP-16+ at addr
'' -- returns 1 (playing), 0 (not playing), or -1 (no response)
'' -- addr is %00..%11

  check := ap16_status(addr) 

  if (check => 0)
    return (check >> 7) & %1                                    ' return status bit (bit7)
  else
    return -1                                                   ' no response
 
    
pub ap16_sfx(addr, sfx, rpts)

'' Plays SFX file on AP-16+ at addr
'' -- addr is %00..%11
'' -- sfx is 0..15
'' -- repeats is 0 for infinite loop, 1..255

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("P")   
  serial.tx("S") 
  serial.tx(sfx)
  serial.tx(rpts)   


pub ap16_aux(addr, aux, rpts)

'' Plays AUX file on AP-16+ at addr
'' -- addr is %00..%11
'' -- aux is 1..8
'' -- repeats is 0 for infinite loop, 1..255

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("P")  
  serial.tx("A")
  serial.tx(aux)
  serial.tx(rpts)


pub ap16_wav(addr, p_str, rpts)

'' Plays WAV file (name at p_str) on AP-16+ at addr
'' -- addr is %00..%11
'' -- p_str is pointer to name of file
'' -- repeats is 0 for infinite loop, 1..255

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("P")  
  serial.tx("W")
  serial.str(p_str)
  serial.tx(13)      
  serial.tx(rpts)

  
pub ap16_volume(addr, left, right)

'' Sets AP-16+ volume levels
'' -- addr is %00..%11
'' -- left and right are levels, 0..100

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("L")
  serial.tx(0 #> left <# 100)  
  serial.tx(0 #> right <# 100)
   

pub ap16_speed(addr, speed)

'' Sets AP-16+ playback speed
'' -- addr is %00..%11
'' -- speed is 50% to 200%

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("S")
  serial.tx(50 #> speed <# 200)


pub ap16_relay(addr, state)

'' Sets AP-16+ relay state
'' -- addr is %00..%11
'' -- state is 0 or 1

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("R")
  serial.tx(state) 
 

con

  { ----------------------------- }
  {  H C - 8 +   R O U T I N E S  } 
  { ----------------------------- }


dat

  HC8           byte    "!HC8", 0


pub hc8_setup

'' Start serial driver on SIO at EFX_BAUD (half-duplex true mode)
'' -- requires serial object defined in project
''    * object must be capable of half-duplex comms over a single wire
''    * assumes board has not been modified for 2-wire serial

  serial.start(SIO_RX, SIO_RX, %1100, EFX_BAUD)
  time.pause(5)
  serial.rxflush


pub hc8_reset(addr)

'' Resets the HC-8+ at addr
'' -- addr is %00..%11
''    * use $FF for all HC-8+s

  serial.str(@HC8)
  serial.tx(addr)
  serial.tx("X")


pub hc8_level(addr, ch, level)

'' Sets channel to level
'' -- addr is %00..%11
'' -- ch is 0..7, "A" or $FF (all)
'' -- level is 0..255

  serial.str(@HC8)
  serial.tx(addr)
  serial.tx("L")
  serial.tx(ch)
  serial.tx(0 #> level <#255)


pub hc8_set_all(addr, level)

'' Sets all channels to level
'' -- addr is %00..%11
'' -- level is 0..255

  hc8_level(addr, "A", level)                                    ' set all to level


pub hc8_randomize_levels(addr, mask)

'' Randomizes levels of channels defined by mask
'' -- addr is %00..%11
'' -- mask is %00000000..%11111111

  serial.str(@HC8)
  serial.tx(addr)
  serial.tx("L")
  serial.tx("R")
  serial.tx(mask)


pub hc8_set_digital(addr, ch, state)

'' Sets channel to level
'' -- addr is %00..%11
'' -- ch is 0..7, "A" or $FF (all)
'' -- state is 0 or 1

  serial.str(@HC8)
  serial.tx(addr)
  serial.tx("L")
  serial.tx(ch)
  serial.tx(state & 1)


pub hc8_digital_select(addr, states, mask)

'' Apply digital control of selected outputs
'' -- addr is %00..%11
'' -- states defines channel states (0 or 1)
'' -- mask defines pins that are affected by command

  serial.str(@HC8)
  serial.tx(addr)
  serial.tx("D")
  serial.tx("S")
  serial.tx(states)
  serial.tx(mask)

  
pub hc8_digital_randomize(addr, mask)

'' Digitally (100% or 0%) randomize selected outputs
'' -- addr is %00..%11
'' -- mask defines pins that are affected by command

  serial.str(@HC8)
  serial.tx(addr)
  serial.tx("D")
  serial.tx("R")
  serial.tx(mask)


pub hc8_fade(addr, ch, lvlstart, lvlstop, ms)

'' Fade channel
'' -- addr is %00..%11
'' -- ch is 0..3
'' -- lvlstart is starting value, 0..255
'' -- lvlstop is ending value, 0..255
'' -- ms is fade time in milliseconds

  serial.str(@HC8)
  serial.tx(addr)
  serial.tx("F")
  serial.tx("H")                                                ' use high resolution fade
  serial.tx(ch)
  serial.tx(0 #> lvlstart <# 255)
  serial.tx(0 #> lvlstop <# 255)
  serial.tx(ms.byte[0])                                         ' send millis Little Endian
  serial.tx(ms.byte[1]) 


pub hc8_crossfade(addr, chA, chB, tenths)

'' Full cross-fade
'' -- addr is %00..%11
'' -- channel chA starts at 100%
'' -- channel chB starts at   0%
'' -- tenths is timing in 0.1s (100ms) units    

  serial.str(@HC8)
  serial.tx(addr)
  serial.tx("F")
  serial.tx("X")
  serial.tx(chA)
  serial.tx(chB) 
  serial.tx(tenths)


pub hc8_read_ttl(addr) | work

'' Reads TTL pins from HC-8+
'' -- addr is %00..%11
'' -- returns %00000000..%11111111, or -1 if bad address or no response

  if ((addr < %00) or (addr > %11))
    return -1

  serial.str(@HC8)
  serial.tx(addr)
  serial.tx("I")
  serial.rxflush

  return serial.rxtime(25)  


con

  { ----------------------------- }
  {  F C - 4 +   R O U T I N E S  }
  { ----------------------------- }


dat

  FC4           byte    "!FC4", 0


pub fc4_setup

'' Start serial driver on SIO at EFX_BAUD (half-duplex true mode)
'' -- requires serial object defined in project
''    * object must be capable of half-duplex comms over a single wire
''    * assumes board has not been modified for 2-wire serial

  serial.start(SIO_RX, SIO_RX, %1100, EFX_BAUD)
  time.pause(5)
  serial.rxflush

  
pub fc4_reset(addr)

'' Sets all FC-4+ outputs at addr to off
'' -- addr is %00..%11
''    * use $FF for all FC-4+s

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("X")


pub fc4_level(addr, ch, level)

'' Sets channel to level
'' -- addr is %00..%11
'' -- ch is 0..3
'' -- level is 0..255

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("L")
  serial.tx(ch)
  serial.tx(0 #> level <# 255)


pub fc4_set_all(addr, level)

'' Sets all channels to level, 0 to 255

  fc4_level(addr, "A", level)                                   ' set all to level


pub fc4_randomize_levels(addr, mask)

'' Randomizes levels of channels defined by mask
'' -- addr is %00..%11
'' -- mask is %0000..%1111

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("L")
  serial.tx("R")
  serial.tx(mask)


pub fc4_set_digital(addr, ch, state)

'' Sets channel to level
'' -- addr is %00..%11
'' -- ch is 0..3, "A" or $FF (all)
'' -- state is 0 or 1

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("L")
  serial.tx(ch)
  serial.tx(state & 1)


pub fc4_digital_select(addr, states, mask)

'' Apply digital control of selected outputs
'' -- addr is %00..%11
'' -- states defines channel states (0 or 1)
'' -- mask defines pins that are affected by command

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("D")
  serial.tx("S")
  serial.tx(states)
  serial.tx(mask)


pub fc4_digital_randomize(addr, mask)

'' Digitally (100% or 0%) randomize selected outputs
'' -- addr is %00..%11
'' -- mask defines pins that are affected by command

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("D")
  serial.tx("R")
  serial.tx(mask)
  

pub fc4_fade(addr, ch, lvlstart, lvlstop, ms)

'' Fade channel
'' -- addr is %00..%11
'' -- ch is 0..3
'' -- lvlstart is starting value, 0..255
'' -- lvlstop is ending value, 0..255
'' -- ms is fade time in milliseconds

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("F")
  serial.tx("H")                                                ' use high resolution fade
  serial.tx(ch)
  serial.tx(0 #> lvlstart <# 255)
  serial.tx(0 #> lvlstop <# 255)
  serial.tx(ms.byte[0])                                         ' send millis Little Endian
  serial.tx(ms.byte[1]) 


pub fc4_crossfade(addr, chA, chB, tenths)

'' Full cross-fade
'' -- addr is %00..%11
'' -- channel chA starts at 100%
'' -- channel chB starts at   0%
'' -- tenths is timing in 0.1s (100ms) units    

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("F")
  serial.tx("X")
  serial.tx(chA)
  serial.tx(chB) 
  serial.tx(tenths)


pub fc4_read_ttl(addr) | work

'' Reads TTL pins from FC-4+
'' -- addr is %00..%11
'' -- returns %00..%11, or -1 if bad address or no response

  if ((addr < %00) or (addr > %11))
    return -1

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("I")
  serial.rxflush

  return serial.rxtime(25)
   

con

  { ----------------------------- }
  {  B A C K G R O U N D   C O G  }
  {  - global milliseconds        }
  {  - R/G LED                    }
  {  - TTL/DMX inputs scanning    }
  { ----------------------------- }
  

var  

  long  bcog                                                    ' background cog #
  long  bcstack[64]                                             ' stack for background cog
                                                                 
  long  millis                                                  ' global milliseconds register
                                                                 
                                                                 
pri background | t                                              ' launch with cognew()
                                                                 
  io.low(R_LED)                                                 ' setup R/G LED pins
  io.low(G_LED)                                                  
                                                                 
  io.high(LD_165)                                               ' setup x165 io pins
  io.low(CLK_165)                                                
  io.input(DO_165)                                               
                                                                 
  millis := 0                                                    
                                                                 
  t := cnt                                                      ' sync loop timer
  repeat                                                         
    waitcnt(t += MS_001)                                        ' run loop every millisecond
    ++millis                                                     
    refresh_rg_led                                              ' update the R/G LED
    scan_inputs                                                 ' re-scan TTL & DMX inputs
                                                                 
                                                                 
var                                                              
                                                                 
  long  rgycolor[2]                                             ' led phase colors
  long  rgytime[2]                                              ' led phase timing (ms)
  long  rgphase                                                 ' current phase (red or green chip)
  long  phasetimer                                              ' time in phase
  long  rgtimer                                                 ' timer for rg process
                                                                 
                                                                 
pri refresh_rg_led                                              ' call only from support()
                                                                 
  if (++phasetimer => rgytime[rgphase])                         ' done with this phase?
    phasetimer := 0                                             ' yes, reset timer
    rgphase := 1 - rgphase                                      '  and invert phase
                                                                 
  if (++rgtimer == 25)                                           
    rgtimer := 0                                                 
                                                                 
  case rgycolor[rgphase]                                        ' set led to color for phase
    OFF:                                                         
      outa[R_LED..G_LED] := %00                                  
                                                                 
    GRN:                                                         
      outa[R_LED..G_LED] := %01                                  
                                                                 
    RED:                                                         
      if (rgtimer < 8)                                           
        outa[R_LED..G_LED] := %10                               ' 10 red
      else                                                       
        outa[R_LED..G_LED] := %00                               ' 15 off (to match green brightness)
                                                                 
    YEL:                                                         
      if (rgtimer == 24)                                         
        outa[R_LED..G_LED] := %10                               '  1 red
      else                                                       
        outa[R_LED..G_LED] := %01                               ' 24 green
                                                                 
                                                                 
pri scan_inputs | tempin                                         
                                                                 
'' Scan TTL and DMX address inputs                               
                                                                 
  outa[LD_165] := 0                                             ' blip Shift/Load line    
  outa[LD_165] := 1                                              
                                                                 
  tempin := 0                                                    
                                                                 
  repeat 16                                                     ' read 16 bits         
    tempin := (tempin << 1) | ina[DO_165]                       ' get new bit             
    outa[CLK_165] := 1                                          ' blip clock              
    outa[CLK_165] := 0                                           
                                                                 
  ttlpins := tempin & $FF                                       ' update global vars
  dmxaddr := (tempin >> 8) | (ina[DMX_A8] << 8) & $1FF                                      
                                                                 
                                                                 
dat { license }

{{

  Copyright (C) 2011-2016 Jon McPhalen    

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      