'' =================================================================================================
''
''   File....... jm_fade_manager.spin 
''   Purpose.... Manages multi-pin fading for dimmer objects
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2012-15 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... 
''   Started.... 
''   Updated.... 17 MAR 2015
''
'' =================================================================================================


con
                                                                  
  SCALE = 50                                                    ' divisor for ms
                                                                  
                                                                  
var                                                               
                                                                  
  long  cog                                                       
                                                                  
  long  looptix                                                 ' tix in bg timing loop
  long  channels                                                ' channels in use (1..8)
  long  lvlspntr                                                ' pointer to current levels
  long  tgtspntr                                                ' pointer to target levels
  long  stmspntr                                                ' pointer to step timing
  long  maskpntr                                                ' pointer to control mask
                                                                 
  long  steptime[8]                                              
  byte  target[8]                                               ' target levels
                                                                 
  byte  ctrlmask                                                ' channels enable bits
                                                                 
  
pub null

  ' This is not a top-level object
  
    
pub start(count, p_levels)

'' Start fade manager cog
'' -- count is the number of channels (1 to 8)
'' -- p_levels is a pointer to the levels array (array of bytes)

  stop

  looptix  := clkfreq / (1000 * SCALE)                          ' set cog loop timing
  channels := 1 #> count <# 8                                   ' set channel count
  lvlspntr := p_levels                                          ' copy pointer to levels array
  tgtspntr := @target                                           ' setup for PASM cog
  stmspntr := @steptime                                          
  maskpntr := @ctrlmask                                          
                                                                 
  cog := cognew(@fademgr, @looptix) + 1                         ' launch fade manager cog
                                                                 
  return cog  


pub stop

'' Stop background cog if already running

  if (cog)
    cogstop(cog - 1)
    cog := 0


pub high(ch)

'' Set channel on (high)

  set(ch, 255)


pub low(ch)

'' Set channel off(low)

  set(ch, 0)


pub toggle(ch)

'' Toggle channel

  if ((ch => 0) and (ch < channels))                            ' good channel?
    if (byte[lvlspntr][ch] => 128)                              ' on? -- 50% or more
      set(ch, 0)                                                '  yes, turn off
    else                                                         
      set(ch, 255)                                              '  no, turn on
                                                                 
                                                                 
pub set(ch, level)                                               
                                                                 
'' Set channel to level without delay                            
                                                                 
  if ((ch => 0) and (ch < channels))                            ' good channel?
    level := 0 #> level <# 255                                   
    ctrlmask &= !(1 << ch)                                      ' disable channel
    byte[lvlspntr][ch] := level                                 ' update levels array
    target[ch] := level                                         ' set new target
    steptime[ch] := 0                                           ' move now
    ctrlmask |= (1 << ch)                                       ' enable channel
                                                                 
                                                                  
pub set_all(level)                                                
                                                                  
'' Set all channels to level without delay                        
                                                                  
  level := 0 #> level <# 255                                    ' validate level
                                                                 
  ctrlmask := %0000_0000                                        ' disable all channels
                                                                 
  bytefill(lvlspntr, level, channels)                           ' set start levels 
  bytefill(@target, level, channels)                            ' set target levels
  longfill(@steptime, 0, channels)                              ' set timing to 0     
                                                                 
  ctrlmask := %1111_1111 >> (8 - channels)                      ' enable available channels
                                                                 

pub set_range(first, last, level)

'' Set all channels in range first..last to level without delay

  first := 0 #> first <# channels-1
  last  := 0 #> last <# channels-1  

  if (last < first)
    result := first
    first := last
    last := result

  level := 0 #> level <# 255                                    ' validate level
                                                                 
  ctrlmask := %0000_0000                                        ' disable all channels
                                                                 
  bytefill(lvlspntr+first, level, (last-first+1))               ' set start levels 
  bytefill(@target+first, level, (last-first+1))                ' set target levels
  longfill(@steptime+first, 0, (last-first+1))                  ' set timing to 0     
                                                                 
  ctrlmask := %1111_1111 >> (8 - channels)                      ' enable available channels
                                                                  
  
pub digital(value, chmask) | idx, level

'' Sets enabled pins (in chmask) to corresponding bit in value

  ctrlmask := !chmask                                           ' disable channels to update
                                                                 
  repeat idx from 0 to channels-1                               ' loop through available channels
    if (chmask & (1 << idx))                                    ' selected channel?
      if (value & (1 << idx))                                   ' if value bit set
        level := 255                                            '   full on
      else                                                      ' else
        level := 0                                              '   off
      byte[lvlspntr][idx] := level                              ' no delay
      target[idx] := level                                       
      steptime[idx] := 0                                         
                                                                 
  ctrlmask := %1111_1111 >> (8 - channels)                      ' enable available channels
                                                                 
  
pub fade(ch, lvlstart, lvlstop, ms)

'' Do fade on channel ch from lvlstart to lvlstop in ms milliseconds

  if ((ch => 0) and (ch < channels) and (ms => 0))              ' good channel
    lvlstart := 0 #> lvlstart <# 255                            ' validate levels
    lvlstop  := 0 #> lvlstop  <# 255                             
                                                                 
    ctrlmask &= !(1 << ch)                                      ' disable channel
                                                                 
    byte[lvlspntr][ch] := lvlstart                              ' set start level
    target[ch] := lvlstop                                       ' set target level
    if (ms == 0)                                                 
      steptime[ch] := 0                                          
    else                                                         
      steptime[ch] := (ms * SCALE) / ||(lvlstop-lvlstart)       ' set timing
                                                                 
    ctrlmask |= (1 << ch)                                       ' enable channel
                                                                 
                                                                 
pub fade_all(lvlstart, lvlstop, ms) | span                       
                                                                 
'' Fade all channels from lvlstart to lvlstop                    
                                                                 
  if (ms => 0)                                                  ' good timing?
    lvlstart := 0 #> lvlstart <# 255                            ' validate levels
    lvlstop  := 0 #> lvlstop  <# 255                             
    span := ||(lvlstop - lvlstart)                              ' calculate span between levels
                                                                 
    ctrlmask := %0000_0000                                      ' disable all channels
                                                                 
    bytefill(lvlspntr, lvlstart, channels)                      ' set start levels  
    bytefill(@target, lvlstop, channels)                        ' set target levels 
    longfill(@steptime, (ms * SCALE) / span, channels)          ' set timing       
                                                                 
    ctrlmask := %1111_1111 >> (8 - channels)                    ' enable all channels
                                                                 
                                                                 
pub cross_fade(ach, bch, ms)                                     
                                                                 
'' Do full cross-fade between ach (0..7) and bch (7..0)          
                                                                 
  if ((ach => 0) and (ach < channels))                          ' a channel good?
    if ((bch => 0) and (bch < channels))                        ' b channel good?
      if (bch <> ach)                                           ' not the same channel?
        ctrlmask &= !((1 << ach) | (1 << bch))                  ' disable selected channels
                                                                 
        byte[lvlspntr][ach] := 255                              ' a starts full bright
        target[ach] := 0                                        ' a ends off
        steptime[ach] := (ms * SCALE) / 255                      
                                                                 
        byte[lvlspntr][bch] := 0                                ' b starts off                         
        target[bch] := 255                                      ' b ends full bright
        steptime[bch] := steptime[ach]                          ' match timing
                                                                 
        ctrlmask |= ((1 << ach) | (1 << bch))                   ' re-enable selected channels
                                                                 
                                                                 
pub at_target(ch)                                                
                                                                  
'' Returns true if channel is at target level                     
                                                                  
  if ((ch => 0) and (ch < channels))                              
    return (byte[lvlspntr][ch] == target[ch])                     
  else                                                            
    return false                                                  
                                                                  
                                                                  
pub read(ch)                                                      
                                                                  
'' Returns channel brightness                                     
                                                                  
  if ((ch => 0) and (ch =< channels))                             
    return byte[lvlspntr][ch]                                     
  else
    return -1


dat

                        org     0

fademgr                 mov     t1, par                         ' start of structure
                        rdlong  tix, t1                         ' get tix for timing loop
                        add     t1, #4                           
                        rdlong  nchans, t1                      ' get # of channels
                        add     t1, #4                           
                        rdlong  hublvls, t1                     ' get hub addr of levels
                        add     t1, #4                           
                        rdlong  hubtgts, t1                     ' get hub addr of targets
                        add     t1, #4                           
                        rdlong  hubstms, t1                     ' get hub addr of step timing
                        add     t1, #4                           
                        rdlong  hubmask, t1                      
                                                                 
starttimer              mov     timer, tix                      ' start loop timer
                        add     timer, cnt                      ' sync with system counter                                                    
                                                                 
fadermain               rdbyte  mask, hubmask                   ' get control mask
                        tjz     mask, #starttimer               ' skip if all are disabled
                                                                 
' ----------------------                                         
                                                                 
ch0                     mov     cpntr, hublvls                  ' reset hub pointers 
                        mov     tpntr, hubtgts                   
                        mov     spntr, hubstms                   
                                                                 
                        test    mask, #(1 << 0)         wc      ' channel enabled?
        if_nc           jmp     #ch1                            ' if no, skip to next                      
                                                                 
                        rdbyte  clevel, cpntr                   ' read current level
                        rdbyte  tlevel, tpntr                   ' read target level
                        rdlong  steptm, spntr                   ' read channel step timing 
                                                                 
                        cmp     clevel, tlevel          wc, wz  ' current == target?   
        if_e            jmp     #ch0exit                        ' if yes we can exit
                                                                 
                        tjnz    steptm, #ch0step                ' step time at 0?
                        wrbyte  tlevel, cpntr                   ' yes, write target to level
                        jmp     #ch0exit                        '  and exit
                                                                 
ch0step                 cmp     ch0steptm, steptm       wc, wz  ' is step timing done?
        if_b            add     ch0steptm, #1                   ' no, update step timer
        if_b            jmp     #ch0exit                        '  and exit
                                                                 
                        cmp     clevel, tlevel          wc, wz  ' check current to target
        if_b            add     clevel, #1                      ' current < target?
        if_a            sub     clevel, #1                      ' current > target?                      
                        wrbyte  clevel, cpntr                   ' update current
                        mov     ch0steptm, #0                   ' restart step timer 
                                                                 
ch0exit                 cmp     nchans, #1              wc, wz  ' check channel count
        if_e            jmp     #looppad                        ' if done, jump to timing pad
                                                                 
' ----------------------                                         
                                                                 
ch1                     add     cpntr, #1                       ' update pointer to ch1
                        add     tpntr, #1                        
                        add     spntr, #4                        
                                                                 
                        test    mask, #(1 << 1)         wc       
        if_nc           jmp     #ch1exit                         
                                                                 
                        rdbyte  tlevel, tpntr                    
                        rdbyte  clevel, cpntr                           
                        rdlong  steptm, spntr                    
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_e            jmp     #ch1exit                         
                                                                 
                        tjnz    steptm, #ch1step                 
                        wrbyte  tlevel, cpntr                    
                        jmp     #ch1exit                         
                                                                 
ch1step                 cmp     ch1steptm, steptm       wc, wz   
        if_b            add     ch1steptm, #1                    
        if_b            jmp     #ch1exit                         
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_a            sub     clevel, #1                       
        if_b            add     clevel, #1                       
                        wrbyte  clevel, cpntr                    
                        mov     ch1steptm, #0                    
                                                                 
ch1exit                 cmp     nchans, #2              wc, wz   
        if_e            jmp     #looppad                         
                                                                 
' ----------------------                                         
                                                                 
ch2                     add     cpntr, #1                        
                        add     tpntr, #1                        
                        add     spntr, #4                        
                                                                 
                        test    mask, #(1 << 2)         wc       
        if_nc           jmp     #ch2exit                         
                                                                 
                        rdbyte  tlevel, tpntr                    
                        rdbyte  clevel, cpntr                           
                        rdlong  steptm, spntr                    
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_e            jmp     #ch2exit                         
                                                                 
                        tjnz    steptm, #ch2step                 
                        wrbyte  tlevel, cpntr                    
                        jmp     #ch2exit                         
                                                                 
ch2step                 cmp     ch2steptm, steptm       wc, wz   
        if_b            add     ch2steptm, #1                    
        if_b            jmp     #ch2exit                         
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_a            sub     clevel, #1                       
        if_b            add     clevel, #1                       
                        wrbyte  clevel, cpntr                    
                        mov     ch2steptm, #0                    
                                                                 
ch2exit                 cmp     nchans, #3              wc, wz   
        if_e            jmp     #looppad                         
                                                                 
' ----------------------                                         
                                                                 
ch3                     add     cpntr, #1                        
                        add     tpntr, #1                        
                        add     spntr, #4                        
                                                                 
                        test    mask, #(1 << 3)         wc       
        if_nc           jmp     #ch3exit                         
                                                                 
                        rdbyte  tlevel, tpntr                    
                        rdbyte  clevel, cpntr                           
                        rdlong  steptm, spntr                    
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_e            jmp     #ch3exit                         
                                                                 
                        tjnz    steptm, #ch3step                 
                        wrbyte  tlevel, cpntr                    
                        jmp     #ch3exit                         
                                                                 
ch3step                 cmp     ch3steptm, steptm       wc, wz   
        if_b            add     ch3steptm, #1                    
        if_b            jmp     #ch3exit                         
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_a            sub     clevel, #1                       
        if_b            add     clevel, #1                       
                        wrbyte  clevel, cpntr                    
                        mov     ch3steptm, #0                    
                                                                 
ch3exit                 cmp     nchans, #4              wc, wz   
        if_e            jmp     #looppad                         
                                                                 
' ----------------------                                         
                                                                 
ch4                     add     cpntr, #1                        
                        add     tpntr, #1                        
                        add     spntr, #4                        
                                                                 
                        test    mask, #(1 << 4)         wc       
        if_nc           jmp     #ch4exit                         
                                                                 
                        rdbyte  tlevel, tpntr                    
                        rdbyte  clevel, cpntr                           
                        rdlong  steptm, spntr                    
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_e            jmp     #ch4exit                         
                                                                 
                        tjnz    steptm, #ch4step                 
                        wrbyte  tlevel, cpntr                    
                        jmp     #ch4exit                         
                                                                 
ch4step                 cmp     ch4steptm, steptm       wc, wz   
        if_b            add     ch4steptm, #1                    
        if_b            jmp     #ch4exit                         
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_a            sub     clevel, #1                       
        if_b            add     clevel, #1                       
                        wrbyte  clevel, cpntr                    
                        mov     ch4steptm, #0                    
                                                                 
ch4exit                 cmp     nchans, #5              wc, wz   
        if_e            jmp     #looppad                         
                                                                 
' ----------------------                                         
                                                                 
ch5                     add     cpntr, #1                        
                        add     tpntr, #1                        
                        add     spntr, #4                        
                                                                 
                        test    mask, #(1 << 5)         wc       
        if_nc           jmp     #ch5exit                         
                                                                 
                        rdbyte  tlevel, tpntr                    
                        rdbyte  clevel, cpntr                           
                        rdlong  steptm, spntr                    
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_e            jmp     #ch5exit                         
                                                                 
                        tjnz    steptm, #ch5step                 
                        wrbyte  tlevel, cpntr                    
                        jmp     #ch5exit                         
                                                                 
ch5step                 cmp     ch5steptm, steptm       wc, wz   
        if_b            add     ch5steptm, #1                    
        if_b            jmp     #ch5exit                         
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_a            sub     clevel, #1                       
        if_b            add     clevel, #1                       
                        wrbyte  clevel, cpntr                    
                        mov     ch5steptm, #0                    
                                                                 
ch5exit                 cmp     nchans, #6              wc, wz   
        if_e            jmp     #looppad                         
                                                                 
' ----------------------                                         
                                                                 
ch6                     add     cpntr, #1                        
                        add     tpntr, #1                        
                        add     spntr, #4                        
                                                                 
                        test    mask, #(1 << 6)         wc       
        if_nc           jmp     #ch6exit                         
                                                                 
                        rdbyte  tlevel, tpntr                    
                        rdbyte  clevel, cpntr                           
                        rdlong  steptm, spntr                    
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_e            jmp     #ch6exit                         
                                                                 
                        tjnz    steptm, #ch6step                 
                        wrbyte  tlevel, cpntr                    
                        jmp     #ch6exit                         
                                                                 
ch6step                 cmp     ch6steptm, steptm       wc, wz   
        if_b            add     ch6steptm, #1                    
        if_b            jmp     #ch6exit                         
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_a            sub     clevel, #1                       
        if_b            add     clevel, #1                       
                        wrbyte  clevel, cpntr                    
                        mov     ch6steptm, #0                    
                                                                 
ch6exit                 cmp     nchans, #7              wc, wz   
        if_e            jmp     #looppad                         
                                                                 
' ----------------------                                         
                                                                 
ch7                     add     cpntr, #1                        
                        add     tpntr, #1                        
                        add     spntr, #4                        
                                                                 
                        test    mask, #(1 << 7)         wc       
        if_nc           jmp     #ch7exit                         
                                                                 
                        rdbyte  tlevel, tpntr                    
                        rdbyte  clevel, cpntr                           
                        rdlong  steptm, spntr                    
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_e            jmp     #ch7exit                         
                                                                 
                        tjnz    steptm, #ch7step                 
                        wrbyte  tlevel, cpntr                    
                        jmp     #ch7exit                         
                                                                 
ch7step                 cmp     ch7steptm, steptm       wc, wz   
        if_b            add     ch7steptm, #1                    
        if_b            jmp     #ch7exit                         
                                                                 
                        cmp     clevel, tlevel          wc, wz   
        if_a            sub     clevel, #1                       
        if_b            add     clevel, #1                       
                        wrbyte  clevel, cpntr                    
                        mov     ch7steptm, #0                    
                                                                 
ch7exit                                                          
                                                                 
' ----------------------                                         
                                                                 
looppad                 waitcnt timer, tix                      ' let loop time expire, reload
                        jmp     #fadermain                       
                                                                 
' --------------------------------------------------------------------------------------------------

ch0steptm               long    0                               ' step timing (cog)
ch1steptm               long    0                                
ch2steptm               long    0                                
ch3steptm               long    0                                
ch4steptm               long    0                                
ch5steptm               long    0                                
ch6steptm               long    0                                
ch7steptm               long    0                                
                                                                 
tix                     res     1                               ' tix in timing loop
nchans                  res     1                               ' number of fader channels
hublvls                 res     1                               ' hub address of dimmer levels
hubtgts                 res     1                               ' hub address of targets
hubstms                 res     1                               ' hub address of step timing
hubmask                 res     1                               ' hub address of control mask
                                                                 
timer                   res     1                               ' loop timer
                                                                 
mask                    res     1                               ' control mask
tpntr                   res     1                               ' target pointer
tlevel                  res     1                               ' target level 
cpntr                   res     1                               ' current pointer
clevel                  res     1                               ' current level
spntr                   res     1                               ' step time pointer
steptm                  res     1                               ' step timing 
                                                                 
t1                      res     1                               ' work vars
t2                      res     1                                
                                                                 
                        fit     496
                                                

dat { license }

{{

  Copyright (C) 2012-15 Jon McPhalen 

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}